#ifndef LIBEASY_QOS_H
#define LIBEASY_QOS_H

/* Needed to get well-sized integers */
#include <stdint.h>

/* Needed to get the IFNAMSIZ define */
#include <net/if.h>

/* For Error logging */
#include <syslog.h>

#ifdef __cplusplus
extern "C" {
#endif

extern const struct qos_ops bcm_rdpa_ops;
extern const struct qos_ops qos_test_ops;

/* TBD: are these structures generic enough to support any QoS stats?
   So far they are derived from the Broadcom queue stats structures, but in the
   future Mediatek networking chipset support would be added too. */

/** Scheduling algorithms possible for a queue */
enum qos_scheduler {
	scheduler_invalid = 0,
	SP,
	WRR,
	SP_WRR,
	WFQ
};

/** This struct represents the QoS statistics of a queue. */
struct qos_stats {
    int32_t tx_packets;
    int32_t tx_bytes;
    int32_t tx_dropped_packets;
    int32_t tx_dropped_bytes;
};

/** Used to configure a queue. */
struct qos_queue_config {
	char iface[IFNAMSIZ];
	enum qos_scheduler scheduler;
	int32_t precedence;
	int32_t rate;
	int32_t burst_size;
	int32_t weight;
};

struct qos_ops {
	/** interface name/prefix to match */
	const char *ifname;

	int (*get_stats)(const char *ifname, int queue_id, struct qos_stats *stats);
};

/* API list */
int qos_get_stats(const char *ifname, int queue_id, struct qos_stats *stats);

#ifdef __cplusplus
}
#endif

#endif // LIBEASY_QOS_H
